@@ -2,18 +2,20 @@ package machinehealthcheck
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"reflect"
 	"strings"
 	"testing"
 	"time"
 
+	. "github.com/onsi/gomega"
 	mapiv1beta1 "github.com/openshift/machine-api-operator/pkg/apis/machine/v1beta1"
 	"github.com/openshift/machine-api-operator/pkg/util/conditions"
 	maotesting "github.com/openshift/machine-api-operator/pkg/util/testing"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/equality"
-	"k8s.io/apimachinery/pkg/api/errors"
+	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
@@ -23,7 +25,6 @@ import (
 	"k8s.io/utils/pointer"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
-	"sigs.k8s.io/controller-runtime/pkg/handler"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 )
 
@@ -31,6 +32,10 @@ const (
 	namespace = "openshift-machine-api"
 )
 
+var (
+	ctx = context.Background()
+)
+
 func init() {
 	// Add types to scheme
 	mapiv1beta1.AddToScheme(scheme.Scheme)
@@ -189,6 +194,8 @@ type expectedReconcile struct {
 }
 
 func TestReconcile(t *testing.T) {
+	ctx := context.Background()
+
 	// healthy node
 	nodeHealthy := maotesting.NewNode("healthy", true)
 	nodeHealthy.Annotations = map[string]string{
@@ -213,7 +220,7 @@ func TestReconcile(t *testing.T) {
 	nodeAnnotatedWithNoExistentMachine.Annotations[machineAnnotationKey] = "annotatedWithNoExistentMachine"
 
 	// node annotated with machine without owner reference
-	nodeAnnotatedWithMachineWithoutOwnerReference := maotesting.NewNode("annotatedWithMachineWithoutOwnerReference", true)
+	nodeAnnotatedWithMachineWithoutOwnerReference := maotesting.NewNode("annotatedWithMachineWithoutOwnerReference", false)
 	nodeAnnotatedWithMachineWithoutOwnerReference.Annotations = map[string]string{
 		machineAnnotationKey: fmt.Sprintf("%s/%s", namespace, "machineWithoutOwnerController"),
 	}
@@ -230,7 +237,11 @@ func TestReconcile(t *testing.T) {
 
 	machineHealthCheck := maotesting.NewMachineHealthCheck("machineHealthCheck")
 	nodeStartupTimeout := 15 * time.Minute
-	machineHealthCheck.Spec.NodeStartupTimeout = nodeStartupTimeout.String()
+	machineHealthCheck.Spec.NodeStartupTimeout = metav1.Duration{Duration: nodeStartupTimeout}
+
+	machineHealthCheckNegativeMaxUnhealthy := maotesting.NewMachineHealthCheck("machineHealthCheckNegativeMaxUnhealthy")
+	negativeOne := intstr.FromInt(-1)
+	machineHealthCheckNegativeMaxUnhealthy.Spec.MaxUnhealthy = &negativeOne
 
 	// remediationExternal
 	nodeUnhealthyForTooLong := maotesting.NewNode("nodeUnhealthyForTooLong", false)
@@ -239,37 +250,70 @@ func TestReconcile(t *testing.T) {
 	}
 	machineUnhealthyForTooLong := maotesting.NewMachine("machineUnhealthyForTooLong", nodeUnhealthyForTooLong.Name)
 
+	nodeAlreadyDeleted := maotesting.NewNode("nodeAlreadyDelete", false)
+	nodeUnhealthyForTooLong.Annotations = map[string]string{
+		machineAnnotationKey: fmt.Sprintf("%s/%s", namespace, "machineAlreadyDeleted"),
+	}
+	machineAlreadyDeleted := maotesting.NewMachine("machineAlreadyDeleted", nodeAlreadyDeleted.Name)
+	machineAlreadyDeleted.SetDeletionTimestamp(&metav1.Time{Time: time.Now()})
+
+	remediationAllowedCondition := mapiv1beta1.Condition{
+		Type:   mapiv1beta1.RemediationAllowedCondition,
+		Status: corev1.ConditionTrue,
+	}
+
 	testCases := []struct {
 		testCase       string
 		machine        *mapiv1beta1.Machine
 		node           *corev1.Node
+		mhc            *mapiv1beta1.MachineHealthCheck
 		expected       expectedReconcile
 		expectedEvents []string
+		expectedStatus *mapiv1beta1.MachineHealthCheckStatus
 	}{
 		{
 			testCase: "machine unhealthy",
 			machine:  machineUnhealthyForTooLong,
 			node:     nodeUnhealthyForTooLong,
+			mhc:      machineHealthCheck,
 			expected: expectedReconcile{
 				result: reconcile.Result{},
 				error:  false,
 			},
 			expectedEvents: []string{EventMachineDeleted},
+			expectedStatus: &mapiv1beta1.MachineHealthCheckStatus{
+				ExpectedMachines:    IntPtr(1),
+				CurrentHealthy:      IntPtr(0),
+				RemediationsAllowed: 0,
+				Conditions: mapiv1beta1.Conditions{
+					remediationAllowedCondition,
+				},
+			},
 		},
 		{
-			testCase: "machine with node unhealthy",
+			testCase: "machine with node healthy",
 			machine:  machineWithNodeHealthy,
 			node:     nodeHealthy,
+			mhc:      machineHealthCheck,
 			expected: expectedReconcile{
 				result: reconcile.Result{},
 				error:  false,
 			},
 			expectedEvents: []string{},
+			expectedStatus: &mapiv1beta1.MachineHealthCheckStatus{
+				ExpectedMachines:    IntPtr(1),
+				CurrentHealthy:      IntPtr(1),
+				RemediationsAllowed: 1,
+				Conditions: mapiv1beta1.Conditions{
+					remediationAllowedCondition,
+				},
+			},
 		},
 		{
 			testCase: "machine with node likely to go unhealthy",
 			machine:  machineWithNodeRecentlyUnhealthy,
 			node:     nodeRecentlyUnhealthy,
+			mhc:      machineHealthCheck,
 			expected: expectedReconcile{
 				result: reconcile.Result{
 					Requeue:      true,
@@ -278,41 +322,77 @@ func TestReconcile(t *testing.T) {
 				error: false,
 			},
 			expectedEvents: []string{EventDetectedUnhealthy},
+			expectedStatus: &mapiv1beta1.MachineHealthCheckStatus{
+				ExpectedMachines:    IntPtr(1),
+				CurrentHealthy:      IntPtr(0),
+				RemediationsAllowed: 0,
+				Conditions: mapiv1beta1.Conditions{
+					remediationAllowedCondition,
+				},
+			},
 		},
 		{
 			testCase: "no target: no machine and bad node annotation",
 			machine:  nil,
 			node:     nodeWithoutMachineAnnotation,
+			mhc:      machineHealthCheck,
 			expected: expectedReconcile{
 				result: reconcile.Result{},
 				error:  false,
 			},
 			expectedEvents: []string{},
+			expectedStatus: &mapiv1beta1.MachineHealthCheckStatus{
+				ExpectedMachines:    IntPtr(0),
+				CurrentHealthy:      IntPtr(0),
+				RemediationsAllowed: 0,
+				Conditions: mapiv1beta1.Conditions{
+					remediationAllowedCondition,
+				},
+			},
 		},
 		{
 			testCase: "no target: no machine",
 			machine:  nil,
 			node:     nodeAnnotatedWithNoExistentMachine,
+			mhc:      machineHealthCheck,
 			expected: expectedReconcile{
 				result: reconcile.Result{},
 				error:  false,
 			},
 			expectedEvents: []string{},
+			expectedStatus: &mapiv1beta1.MachineHealthCheckStatus{
+				ExpectedMachines:    IntPtr(0),
+				CurrentHealthy:      IntPtr(0),
+				RemediationsAllowed: 0,
+				Conditions: mapiv1beta1.Conditions{
+					remediationAllowedCondition,
+				},
+			},
 		},
 		{
 			testCase: "machine no controller owner",
 			machine:  machineWithoutOwnerController,
 			node:     nodeAnnotatedWithMachineWithoutOwnerReference,
+			mhc:      machineHealthCheck,
 			expected: expectedReconcile{
 				result: reconcile.Result{},
 				error:  false,
 			},
-			expectedEvents: []string{},
+			expectedEvents: []string{EventSkippedNoController},
+			expectedStatus: &mapiv1beta1.MachineHealthCheckStatus{
+				ExpectedMachines:    IntPtr(1),
+				CurrentHealthy:      IntPtr(0),
+				RemediationsAllowed: 0,
+				Conditions: mapiv1beta1.Conditions{
+					remediationAllowedCondition,
+				},
+			},
 		},
 		{
 			testCase: "machine no noderef",
 			machine:  machineWithoutNodeRef,
 			node:     nodeAnnotatedWithMachineWithoutNodeReference,
+			mhc:      machineHealthCheck,
 			expected: expectedReconcile{
 				result: reconcile.Result{
 					RequeueAfter: nodeStartupTimeout,
@@ -320,13 +400,86 @@ func TestReconcile(t *testing.T) {
 				error: false,
 			},
 			expectedEvents: []string{EventDetectedUnhealthy},
+			expectedStatus: &mapiv1beta1.MachineHealthCheckStatus{
+				ExpectedMachines:    IntPtr(1),
+				CurrentHealthy:      IntPtr(0),
+				RemediationsAllowed: 0,
+				Conditions: mapiv1beta1.Conditions{
+					remediationAllowedCondition,
+				},
+			},
+		},
+		{
+			testCase: "machine already deleted",
+			machine:  machineAlreadyDeleted,
+			node:     nodeAlreadyDeleted,
+			mhc:      machineHealthCheck,
+			expected: expectedReconcile{
+				result: reconcile.Result{},
+				error:  false,
+			},
+			expectedEvents: []string{},
+			expectedStatus: &mapiv1beta1.MachineHealthCheckStatus{
+				ExpectedMachines:    IntPtr(1),
+				CurrentHealthy:      IntPtr(0),
+				RemediationsAllowed: 0,
+				Conditions: mapiv1beta1.Conditions{
+					remediationAllowedCondition,
+				},
+			},
+		},
+		{
+			testCase: "machine healthy with MHC negative maxUnhealthy",
+			machine:  machineWithNodeHealthy,
+			node:     nodeHealthy,
+			mhc:      machineHealthCheckNegativeMaxUnhealthy,
+			expected: expectedReconcile{
+				result: reconcile.Result{},
+				error:  false,
+			},
+			expectedEvents: []string{},
+			expectedStatus: &mapiv1beta1.MachineHealthCheckStatus{
+				ExpectedMachines:    IntPtr(1),
+				CurrentHealthy:      IntPtr(1),
+				RemediationsAllowed: 0,
+				Conditions: mapiv1beta1.Conditions{
+					remediationAllowedCondition,
+				},
+			},
+		},
+		{
+			testCase: "machine unhealthy with MHC negative maxUnhealthy",
+			machine:  machineUnhealthyForTooLong,
+			node:     nodeUnhealthyForTooLong,
+			mhc:      machineHealthCheckNegativeMaxUnhealthy,
+			expected: expectedReconcile{
+				result: reconcile.Result{
+					Requeue: true,
+				},
+				error: false,
+			},
+			expectedEvents: []string{EventRemediationRestricted},
+			expectedStatus: &mapiv1beta1.MachineHealthCheckStatus{
+				ExpectedMachines:    IntPtr(1),
+				CurrentHealthy:      IntPtr(0),
+				RemediationsAllowed: 0,
+				Conditions: mapiv1beta1.Conditions{
+					{
+						Type:     mapiv1beta1.RemediationAllowedCondition,
+						Status:   corev1.ConditionFalse,
+						Severity: mapiv1beta1.ConditionSeverityWarning,
+						Reason:   mapiv1beta1.TooManyUnhealthyReason,
+						Message:  "Remediation is not allowed, the number of not started or unhealthy machines exceeds maxUnhealthy (total: 1, unhealthy: 1, maxUnhealthy: -1)",
+					},
+				},
+			},
 		},
 	}
 
 	for _, tc := range testCases {
 		t.Run(tc.testCase, func(t *testing.T) {
 			var objects []runtime.Object
-			objects = append(objects, machineHealthCheck)
+			objects = append(objects, tc.mhc)
 			if tc.machine != nil {
 				objects = append(objects, tc.machine)
 			}
@@ -336,11 +489,11 @@ func TestReconcile(t *testing.T) {
 
 			request := reconcile.Request{
 				NamespacedName: types.NamespacedName{
-					Namespace: machineHealthCheck.GetNamespace(),
-					Name:      machineHealthCheck.GetName(),
+					Namespace: tc.mhc.GetNamespace(),
+					Name:      tc.mhc.GetName(),
 				},
 			}
-			result, err := r.Reconcile(request)
+			result, err := r.Reconcile(ctx, request)
 			assertEvents(t, tc.testCase, tc.expectedEvents, recorder.Events)
 			if tc.expected.error != (err != nil) {
 				var errorExpectation string
@@ -361,6 +514,13 @@ func TestReconcile(t *testing.T) {
 					t.Errorf("Test case: %s. Expected: %v, got: %v", tc.node.Name, tc.expected.result, result)
 				}
 			}
+
+			if tc.expectedStatus != nil {
+				g := NewWithT(t)
+				mhc := &mapiv1beta1.MachineHealthCheck{}
+				g.Expect(r.client.Get(ctx, request.NamespacedName, mhc)).To(Succeed())
+				g.Expect(&mhc.Status).To(MatchMachineHealthCheckStatus(tc.expectedStatus))
+			}
 		})
 	}
 }
@@ -623,11 +783,7 @@ func TestMHCRequestsFromMachine(t *testing.T) {
 				objects = append(objects, runtime.Object(tc.mhcs[i]))
 			}
 
-			o := handler.MapObject{
-				Meta:   tc.machine.GetObjectMeta(),
-				Object: tc.machine,
-			}
-			requests := newFakeReconciler(objects...).mhcRequestsFromMachine(o)
+			requests := newFakeReconciler(objects...).mhcRequestsFromMachine(tc.machine)
 			if !reflect.DeepEqual(requests, tc.expectedRequests) {
 				t.Errorf("Expected: %v, got: %v", tc.expectedRequests, requests)
 			}
@@ -824,11 +980,7 @@ func TestMHCRequestsFromNode(t *testing.T) {
 				objects = append(objects, runtime.Object(tc.mhcs[i]))
 			}
 
-			o := handler.MapObject{
-				Meta:   tc.node.GetObjectMeta(),
-				Object: tc.node,
-			}
-			requests := newFakeReconciler(objects...).mhcRequestsFromNode(o)
+			requests := newFakeReconciler(objects...).mhcRequestsFromNode(tc.node)
 			if !reflect.DeepEqual(requests, tc.expectedRequests) {
 				t.Errorf("Expected: %v, got: %v", tc.expectedRequests, requests)
 			}
@@ -1365,12 +1517,12 @@ func TestNeedsRemediation(t *testing.T) {
 							{
 								Type:    "Ready",
 								Status:  "Unknown",
-								Timeout: "300s",
+								Timeout: metav1.Duration{Duration: 300 * time.Second},
 							},
 							{
 								Type:    "Ready",
 								Status:  "False",
-								Timeout: "300s",
+								Timeout: metav1.Duration{Duration: 300 * time.Second},
 							},
 						},
 					},
@@ -1405,12 +1557,12 @@ func TestNeedsRemediation(t *testing.T) {
 							{
 								Type:    "Ready",
 								Status:  "Unknown",
-								Timeout: "300s",
+								Timeout: metav1.Duration{Duration: 300 * time.Second},
 							},
 							{
 								Type:    "Ready",
 								Status:  "False",
-								Timeout: "300s",
+								Timeout: metav1.Duration{Duration: 300 * time.Second},
 							},
 						},
 					},
@@ -1458,12 +1610,12 @@ func TestNeedsRemediation(t *testing.T) {
 							{
 								Type:    "Ready",
 								Status:  "Unknown",
-								Timeout: "300s",
+								Timeout: metav1.Duration{Duration: 300 * time.Second},
 							},
 							{
 								Type:    "Ready",
 								Status:  "False",
-								Timeout: "300s",
+								Timeout: metav1.Duration{Duration: 300 * time.Second},
 							},
 						},
 					},
@@ -1533,12 +1685,12 @@ func TestNeedsRemediation(t *testing.T) {
 							{
 								Type:    "Ready",
 								Status:  "Unknown",
-								Timeout: "300s",
+								Timeout: metav1.Duration{Duration: 300 * time.Second},
 							},
 							{
 								Type:    "Ready",
 								Status:  "False",
-								Timeout: "300s",
+								Timeout: metav1.Duration{Duration: 300 * time.Second},
 							},
 						},
 					},
@@ -1586,12 +1738,12 @@ func TestNeedsRemediation(t *testing.T) {
 							{
 								Type:    "Ready",
 								Status:  "Unknown",
-								Timeout: "300s",
+								Timeout: metav1.Duration{Duration: 300 * time.Second},
 							},
 							{
 								Type:    "Ready",
 								Status:  "False",
-								Timeout: "300s",
+								Timeout: metav1.Duration{Duration: 300 * time.Second},
 							},
 						},
 					},
@@ -1661,12 +1813,12 @@ func TestNeedsRemediation(t *testing.T) {
 							{
 								Type:    "Ready",
 								Status:  "Unknown",
-								Timeout: "300s",
+								Timeout: metav1.Duration{Duration: 300 * time.Second},
 							},
 							{
 								Type:    "Ready",
 								Status:  "False",
-								Timeout: "300s",
+								Timeout: metav1.Duration{Duration: 300 * time.Second},
 							},
 						},
 					},
@@ -1678,81 +1830,6 @@ func TestNeedsRemediation(t *testing.T) {
 			expectedNextCheck:           time.Duration(1 * time.Minute), // 300-200 rounded
 			expectedError:               false,
 		},
-		{
-			testCase: "error bad conditions timeout",
-			target: &target{
-				Machine: mapiv1beta1.Machine{
-					TypeMeta: metav1.TypeMeta{Kind: "Machine"},
-					ObjectMeta: metav1.ObjectMeta{
-						Annotations:     make(map[string]string),
-						Name:            "machine",
-						Namespace:       namespace,
-						Labels:          map[string]string{"foo": "bar"},
-						OwnerReferences: []metav1.OwnerReference{{Kind: "MachineSet"}},
-					},
-					Spec: mapiv1beta1.MachineSpec{},
-					Status: mapiv1beta1.MachineStatus{
-						LastUpdated: &metav1.Time{Time: time.Now().Add(time.Duration(-defaultNodeStartupTimeout) - 1*time.Second)},
-					},
-				},
-				Node: &corev1.Node{
-					ObjectMeta: metav1.ObjectMeta{
-						Name:      "node",
-						Namespace: metav1.NamespaceNone,
-						Annotations: map[string]string{
-							machineAnnotationKey: fmt.Sprintf("%s/%s", namespace, "machine"),
-						},
-						Labels: map[string]string{},
-						UID:    "uid",
-					},
-					TypeMeta: metav1.TypeMeta{
-						Kind: "Node",
-					},
-					Status: corev1.NodeStatus{
-						Conditions: []corev1.NodeCondition{
-							{
-								Type:               corev1.NodeReady,
-								Status:             corev1.ConditionFalse,
-								LastTransitionTime: metav1.Time{Time: time.Now().Add(time.Duration(-200) * time.Second)},
-							},
-						},
-					},
-				},
-				MHC: mapiv1beta1.MachineHealthCheck{
-					ObjectMeta: metav1.ObjectMeta{
-						Name:      "test",
-						Namespace: namespace,
-					},
-					TypeMeta: metav1.TypeMeta{
-						Kind: "MachineHealthCheck",
-					},
-					Spec: mapiv1beta1.MachineHealthCheckSpec{
-						Selector: metav1.LabelSelector{
-							MatchLabels: map[string]string{
-								"foo": "bar",
-							},
-						},
-						UnhealthyConditions: []mapiv1beta1.UnhealthyCondition{
-							{
-								Type:    "Ready",
-								Status:  "Unknown",
-								Timeout: "badTimeout",
-							},
-							{
-								Type:    "Ready",
-								Status:  "False",
-								Timeout: "300s",
-							},
-						},
-					},
-					Status: mapiv1beta1.MachineHealthCheckStatus{},
-				},
-			},
-			timeoutForMachineToHaveNode: defaultNodeStartupTimeout,
-			expectedNeedsRemediation:    false,
-			expectedNextCheck:           time.Duration(0),
-			expectedError:               true,
-		},
 	}
 
 	for _, tc := range testCases {
@@ -1977,7 +2054,7 @@ func TestRemediate(t *testing.T) {
 			err := r.client.Get(context.TODO(), namespacedName(&tc.target.Machine), machine)
 			if tc.deletion {
 				if err != nil {
-					if !errors.IsNotFound(err) {
+					if !apierrors.IsNotFound(err) {
 						t.Errorf("Expected not found error, got: %v", err)
 					}
 				} else {
@@ -1995,10 +2072,11 @@ func TestRemediate(t *testing.T) {
 
 func TestReconcileStatus(t *testing.T) {
 	testCases := []struct {
-		testCase       string
-		mhc            *mapiv1beta1.MachineHealthCheck
-		totalTargets   int
-		currentHealthy int
+		testCase            string
+		mhc                 *mapiv1beta1.MachineHealthCheck
+		totalTargets        int
+		currentHealthy      int
+		remediationsAllowed int32
 	}{
 		{
 			testCase: "status gets new values",
@@ -2015,8 +2093,49 @@ func TestReconcileStatus(t *testing.T) {
 				},
 				Status: mapiv1beta1.MachineHealthCheckStatus{},
 			},
-			totalTargets:   10,
-			currentHealthy: 5,
+			totalTargets:        10,
+			currentHealthy:      5,
+			remediationsAllowed: 5,
+		},
+		{
+			testCase: "when the unhealthy machines exceed maxUnhealthy",
+			mhc: &mapiv1beta1.MachineHealthCheck{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "test",
+					Namespace: namespace,
+				},
+				TypeMeta: metav1.TypeMeta{
+					Kind: "MachineHealthCheck",
+				},
+				Spec: mapiv1beta1.MachineHealthCheckSpec{
+					Selector:     metav1.LabelSelector{},
+					MaxUnhealthy: &intstr.IntOrString{Type: intstr.String, StrVal: "40%"},
+				},
+				Status: mapiv1beta1.MachineHealthCheckStatus{},
+			},
+			totalTargets:        10,
+			currentHealthy:      5,
+			remediationsAllowed: 0,
+		},
+		{
+			testCase: "when the unhealthy machines does not exceed maxUnhealthy",
+			mhc: &mapiv1beta1.MachineHealthCheck{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "test",
+					Namespace: namespace,
+				},
+				TypeMeta: metav1.TypeMeta{
+					Kind: "MachineHealthCheck",
+				},
+				Spec: mapiv1beta1.MachineHealthCheckSpec{
+					Selector:     metav1.LabelSelector{},
+					MaxUnhealthy: &intstr.IntOrString{Type: intstr.String, StrVal: "40%"},
+				},
+				Status: mapiv1beta1.MachineHealthCheckStatus{},
+			},
+			totalTargets:        10,
+			currentHealthy:      7,
+			remediationsAllowed: 1,
 		},
 	}
 	for _, tc := range testCases {
@@ -2025,7 +2144,12 @@ func TestReconcileStatus(t *testing.T) {
 			objects = append(objects, runtime.Object(tc.mhc))
 			r := newFakeReconciler(objects...)
 
-			if err := r.reconcileStatus(tc.mhc, tc.totalTargets, tc.currentHealthy); err != nil {
+			mergeBase := client.MergeFrom(tc.mhc.DeepCopy())
+
+			tc.mhc.Status.ExpectedMachines = &tc.totalTargets
+			tc.mhc.Status.CurrentHealthy = &tc.currentHealthy
+
+			if err := r.reconcileStatus(mergeBase, tc.mhc); err != nil {
 				t.Fatalf("Unexpected error: %v", err)
 			}
 			mhc := &mapiv1beta1.MachineHealthCheck{}
@@ -2038,6 +2162,9 @@ func TestReconcileStatus(t *testing.T) {
 			if *mhc.Status.CurrentHealthy != tc.currentHealthy {
 				t.Errorf("Case: %v. Got: %v, expected: %v", tc.testCase, mhc.Status.CurrentHealthy, tc.currentHealthy)
 			}
+			if mhc.Status.RemediationsAllowed != tc.remediationsAllowed {
+				t.Errorf("Case: %v. Got: %v, expected: %v", tc.testCase, mhc.Status.RemediationsAllowed, tc.remediationsAllowed)
+			}
 		})
 	}
 }
@@ -2110,12 +2237,12 @@ func TestHealthCheckTargets(t *testing.T) {
 								{
 									Type:    "Ready",
 									Status:  "Unknown",
-									Timeout: "300s",
+									Timeout: metav1.Duration{Duration: 300 * time.Second},
 								},
 								{
 									Type:    "Ready",
 									Status:  "False",
-									Timeout: "300s",
+									Timeout: metav1.Duration{Duration: 300 * time.Second},
 								},
 							},
 						},
@@ -2176,12 +2303,12 @@ func TestHealthCheckTargets(t *testing.T) {
 								{
 									Type:    "Ready",
 									Status:  "Unknown",
-									Timeout: "300s",
+									Timeout: metav1.Duration{Duration: 300 * time.Second},
 								},
 								{
 									Type:    "Ready",
 									Status:  "False",
-									Timeout: "300s",
+									Timeout: metav1.Duration{Duration: 300 * time.Second},
 								},
 							},
 						},
@@ -2246,12 +2373,12 @@ func TestHealthCheckTargets(t *testing.T) {
 								{
 									Type:    "Ready",
 									Status:  "Unknown",
-									Timeout: "300s",
+									Timeout: metav1.Duration{Duration: 300 * time.Second},
 								},
 								{
 									Type:    "Ready",
 									Status:  "False",
-									Timeout: "300s",
+									Timeout: metav1.Duration{Duration: 300 * time.Second},
 								},
 							},
 						},
@@ -2299,12 +2426,12 @@ func TestHealthCheckTargets(t *testing.T) {
 								{
 									Type:    "Ready",
 									Status:  "Unknown",
-									Timeout: "300s",
+									Timeout: metav1.Duration{Duration: 300 * time.Second},
 								},
 								{
 									Type:    "Ready",
 									Status:  "False",
-									Timeout: "300s",
+									Timeout: metav1.Duration{Duration: 300 * time.Second},
 								},
 							},
 						},
@@ -2365,12 +2492,12 @@ func TestHealthCheckTargets(t *testing.T) {
 								{
 									Type:    "Ready",
 									Status:  "Unknown",
-									Timeout: "300s",
+									Timeout: metav1.Duration{Duration: 300 * time.Second},
 								},
 								{
 									Type:    "Ready",
 									Status:  "False",
-									Timeout: "300s",
+									Timeout: metav1.Duration{Duration: 300 * time.Second},
 								},
 							},
 						},
@@ -2408,6 +2535,7 @@ func TestHealthCheckTargets(t *testing.T) {
 func TestIsAllowedRemediation(t *testing.T) {
 	// short circuit if ever more than 2 out of 5 go unhealthy
 	maxUnhealthyInt := intstr.FromInt(2)
+	maxUnhealthyNegative := intstr.FromInt(-2)
 	maxUnhealthyString := intstr.FromString("40%")
 	maxUnhealthyIntInString := intstr.FromString("2")
 	maxUnhealthyMixedString := intstr.FromString("foo%50")
@@ -2459,6 +2587,27 @@ func TestIsAllowedRemediation(t *testing.T) {
 			},
 			expected: false,
 		},
+		{
+			testCase: "maxUnhealthy is negative",
+			mhc: &mapiv1beta1.MachineHealthCheck{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "test",
+					Namespace: namespace,
+				},
+				TypeMeta: metav1.TypeMeta{
+					Kind: "MachineHealthCheck",
+				},
+				Spec: mapiv1beta1.MachineHealthCheckSpec{
+					Selector:     metav1.LabelSelector{},
+					MaxUnhealthy: &maxUnhealthyNegative,
+				},
+				Status: mapiv1beta1.MachineHealthCheckStatus{
+					ExpectedMachines: IntPtr(5),
+					CurrentHealthy:   IntPtr(2),
+				},
+			},
+			expected: false,
+		},
 		{
 			testCase: "not above maxUnhealthy (percentange)",
 			mhc: &mapiv1beta1.MachineHealthCheck{
@@ -2596,48 +2745,71 @@ func TestIsAllowedRemediation(t *testing.T) {
 	}
 }
 
-func TestGetNodeStartupTimeout(t *testing.T) {
+func TestGetMaxUnhealthy(t *testing.T) {
 	testCases := []struct {
-		name            string
-		timeout         string
-		expectError     bool
-		expectedTimeout time.Duration
+		name                 string
+		maxUnhealthy         *intstr.IntOrString
+		expectedMaxUnhealthy int
+		expectedMachines     int
+		expectedErr          error
 	}{
 		{
-			name:            "when the timeout is not set",
-			timeout:         "",
-			expectError:     false,
-			expectedTimeout: defaultNodeStartupTimeout,
+			name:                 "when maxUnhealthy is nil",
+			maxUnhealthy:         nil,
+			expectedMaxUnhealthy: 7,
+			expectedMachines:     7,
+			expectedErr:          nil,
 		},
 		{
-			name:            "when the timeout is not parsable",
-			timeout:         "a",
-			expectError:     true,
-			expectedTimeout: time.Duration(0),
+			name:                 "when maxUnhealthy is not an int or percentage",
+			maxUnhealthy:         &intstr.IntOrString{Type: intstr.String, StrVal: "abcdef"},
+			expectedMaxUnhealthy: 0,
+			expectedMachines:     3,
+			expectedErr:          errors.New("invalid value for IntOrString: invalid value \"abcdef\": strconv.Atoi: parsing \"abcdef\": invalid syntax"),
 		},
 		{
-			name:            "when the timeout is  parsable",
-			timeout:         "10s",
-			expectError:     false,
-			expectedTimeout: 10 * time.Second,
+			name:                 "when maxUnhealthy is an int",
+			maxUnhealthy:         &intstr.IntOrString{Type: intstr.Int, IntVal: 3},
+			expectedMachines:     2,
+			expectedMaxUnhealthy: 3,
+			expectedErr:          nil,
+		},
+		{
+			name:                 "when maxUnhealthy is a 40% (of 5)",
+			maxUnhealthy:         &intstr.IntOrString{Type: intstr.String, StrVal: "40%"},
+			expectedMachines:     5,
+			expectedMaxUnhealthy: 2,
+			expectedErr:          nil,
+		},
+		{
+			name:                 "when maxUnhealthy is a 60% (of 7)",
+			maxUnhealthy:         &intstr.IntOrString{Type: intstr.String, StrVal: "60%"},
+			expectedMachines:     7,
+			expectedMaxUnhealthy: 4,
+			expectedErr:          nil,
 		},
 	}
 
 	for _, tc := range testCases {
 		t.Run(tc.name, func(t *testing.T) {
+			g := NewWithT(t)
+
 			mhc := &mapiv1beta1.MachineHealthCheck{
 				Spec: mapiv1beta1.MachineHealthCheckSpec{
-					NodeStartupTimeout: tc.timeout,
+					MaxUnhealthy: tc.maxUnhealthy,
+				},
+				Status: mapiv1beta1.MachineHealthCheckStatus{
+					ExpectedMachines: &tc.expectedMachines,
 				},
 			}
 
-			timeout, err := getNodeStartupTimeout(mhc)
-			if tc.expectError != (err != nil) {
-				t.Errorf("Case: %s. Got: %v, expected: %v", tc.name, err, tc.expectError)
-			}
-			if timeout != tc.expectedTimeout {
-				t.Errorf("Case: %s. Got: %v, expected: %v", tc.name, timeout.String(), tc.expectedTimeout.String())
+			maxUnhealthy, err := getMaxUnhealthy(mhc)
+			if tc.expectedErr != nil {
+				g.Expect(err).To(Equal(tc.expectedErr))
+			} else {
+				g.Expect(err).ToNot(HaveOccurred())
 			}
+			g.Expect(maxUnhealthy).To(Equal(tc.expectedMaxUnhealthy))
 		})
 	}
 }
