@@ -27,8 +27,8 @@ import (
 	machinev1 "github.com/openshift/machine-api-operator/pkg/apis/machine/v1beta1"
 	"k8s.io/client-go/kubernetes/scheme"
 	"k8s.io/client-go/rest"
-	"k8s.io/klog"
-	"k8s.io/klog/klogr"
+	"k8s.io/klog/v2"
+	"k8s.io/klog/v2/klogr"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/envtest"
 	"sigs.k8s.io/controller-runtime/pkg/envtest/printer"
@@ -53,7 +53,6 @@ var (
 	k8sClient  client.Client
 	testEnv    *envtest.Environment
 	mgr        manager.Manager
-	doneMgr    = make(chan struct{})
 	ctx        = context.Background()
 	reconciler *ReconcileMachineSet
 )
